/*
 * Copyright (c) 2018 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <glib.h>
#include <gio/gio.h>

#include <user-awareness.h>

#include "uat-menu.h"
#include "uat-common.h"

extern ua_monitor_h g_ua_mon_h; /**< Monitor handle */
ua_user_h g_user_h = NULL; /**< User handle */
extern ua_service_h g_service_h; /**< Service handle */

char g_user_account_str[MENU_DATA_SIZE + 1] = { "default@default.com" };
char g_user_name_str[MENU_DATA_SIZE + 1] = { "default" };

extern char g_service_str[MENU_DATA_SIZE + 1];
static char g_account_string[MENU_DATA_SIZE + 1] = {0};

GSList *user_list = NULL; /**< User List */
static int idx_user = 0; /**< Found device sequence number */

static void update_user_info(void)
{
	int ret = UA_ERROR_NONE;
	char *account = NULL;
	char *name = NULL;

	if (1 == idx_user) {
		if (g_user_h) {
			ua_user_destroy(g_user_h);
			g_user_h = NULL;
		}

		ua_user_get_default_user(&g_user_h);
	}

	ret  = ua_user_get_account(g_user_h, &account);
	if (UA_ERROR_NONE == ret) {
		memset(g_user_account_str, 0, MENU_DATA_SIZE + 1);
		memcpy(g_user_account_str, account, MENU_DATA_SIZE);
		g_free(account);
	}
	ret  = ua_user_get_name(g_user_h, &name);
	if (UA_ERROR_NONE == ret) {
		memset(g_user_name_str, 0, MENU_DATA_SIZE + 1);
		memcpy(g_user_name_str, name, MENU_DATA_SIZE);
		g_free(name);
	}
}

static void  ___free_users(gpointer data)
{
	ua_user_h handle = data;
	ua_user_destroy(handle);
	handle = NULL;
}

static void __clear_user_list(void)
{
	g_slist_free_full(user_list, ___free_users);
	user_list = NULL;
	idx_user = 0;
}

static bool __foreach_registered_user_cb(
	ua_user_h user_handle, void *user_data)
{
	int ret = UA_ERROR_NONE;

	char *account = NULL;
	char *name = NULL;
	long timestamp = 0;

	if (user_handle) {

		user_list = g_slist_append(user_list, user_handle);
		idx_user++;

		ret = ua_user_get_account(user_handle, &account);
		if (UA_ERROR_NONE == ret) {
			msgb("[%d]", idx_user);
			msgb("User account info %s", account);
			g_free(account);
		}

		ret = ua_user_get_name(user_handle, &name);
		if (UA_ERROR_NONE == ret) {
			msgb("User Name : %s", name);
			g_free(name);
		}

		ret = ua_user_get_last_presence(user_handle, &timestamp);
		if (UA_ERROR_NONE == ret)
			msgb("last present at %ld", timestamp);
	}

	return true;
}

static bool __service_foreach_registered_user_cb(
	ua_service_h service_tmp, ua_user_h user_handle, void *user_data)
{
	int ret = UA_ERROR_NONE;

	char *service_name = NULL;
	char *account = NULL;
	char *name = NULL;

	ret = ua_service_get_name(service_tmp, &service_name);
	if (UA_ERROR_NONE == ret)
		msgb("CB: Service name : %s", service_name);

	g_free(service_name);

	if (user_handle) {

		user_list = g_slist_append(user_list, user_handle);
		idx_user++;

		ret = ua_user_get_account(user_handle, &account);
		if (UA_ERROR_NONE == ret) {
			msgb("[%d]", idx_user);
			msgb("User account info %s", account);
			g_free(account);
		}
		ret = ua_user_get_name(user_handle, &name);
		if (UA_ERROR_NONE == ret) {
			msgb("User Name : %s", name);
			g_free(name);
		}
	}

	return true;
}

static int run_ua_get_default_user(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_user_get_default_user");

	if (g_user_h) {
		ua_user_destroy(g_user_h);
		g_user_h = NULL;
	}

	ret = ua_user_get_default_user(&g_user_h);
	if (UA_ERROR_NONE == ret)
		update_user_info();

	msg(" - ua_get_default_user() ret: [0x%X] [%s]",
		ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_user_create(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_user_create");

	if (g_user_h) {
		ua_user_destroy((g_user_h));
		g_user_h = NULL;
	}

	ret = ua_user_create(&g_user_h);

	msg(" - ua_user_create() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_user_destroy(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_user_destroy");

	if (g_user_h) {
		ret = ua_user_destroy((g_user_h));
		g_user_h = NULL;
	}

	msg(" - ua_user_destroy() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_user_set_account(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_user_set_account");

	check_if(NULL == g_user_h);

	ret = ua_user_set_account(g_user_h, g_user_account_str);

	msg(" - ua_user_set_account() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_user_set_name(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_user_set_name");

	check_if(NULL == g_user_h);

	ret = ua_user_set_name(g_user_h, g_user_name_str);

	msg(" - ua_user_set_name() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_user_add(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_user_add");

	check_if(NULL == g_user_h);

	ret = ua_user_add(g_user_h);

	msg(" - ua_user_add() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_user_remove(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_user_remove");

	check_if(NULL == g_user_h);

	ret = ua_user_remove(g_user_h);

	msg(" - ua_user_remove() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_user_foreach(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	msg("ua_user_foreach_added");

	__clear_user_list();
	ret = ua_user_foreach_added(__foreach_registered_user_cb, NULL);

	msg(" - ua_user_foreach_added() ret: [0x%X] [%s]",
				ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_foreach_added_users(
	MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	msg("ua_service_foreach_added_users");

	__clear_user_list();
	ret = ua_service_foreach_added_users(g_service_h,
				__service_foreach_registered_user_cb, NULL);

	msg(" - ua_service_foreach_added_users() ret: [0x%X] [%s]",
				ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}



static struct menu_data menu_ua_user_account[] = {
	{ "1", "account",
		NULL, NULL, g_user_account_str },
	{ "2", "run", NULL,
		run_ua_user_set_account, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_user_name[] = {
	{ "1", "name",
		NULL, NULL, g_user_name_str },
	{ "2", "run", NULL,
		run_ua_user_set_name, NULL },
	{ NULL, NULL, },
};

static int run_ua_user_get_handle_by_account(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;
	ua_user_h user_handle;

	msg("ua_user_get_handle_by_account");

	ret = ua_user_get_handle_by_account(g_account_string, &user_handle);

	msg(" - ua_user_get_handle_by_account() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	if (UA_ERROR_NONE == ret) {
		msgb("User Handle [%p]", user_handle);
		ua_user_destroy(g_user_h);
		g_user_h = user_handle;
		update_user_info();
	}
	return RET_SUCCESS;
}

static struct menu_data menu_ua_user_get_handle_by_account[] = {
	{ "1", "account",
		NULL, NULL, g_account_string},
	{ "2", "run", NULL,
		run_ua_user_get_handle_by_account, NULL },
	{ NULL, NULL, },
};

static int run_ua_service_add_user(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_add_user");

	check_if('\0' == g_user_account_str[0]);
	check_if(NULL == g_user_h);
	check_if('\0' == g_service_str[0]);
	check_if(NULL == g_service_h);

	ret = ua_service_add_user(g_service_h, g_user_h);

	msg(" - ua_service_add_user() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static int run_ua_service_remove_user(MManager *mm, struct menu_data *menu)
{
	int ret = UA_ERROR_NONE;

	msg("ua_service_remove_user");

	check_if('\0' == g_user_account_str[0]);
	check_if(NULL == g_user_h);
	check_if('\0' == g_service_str[0]);
	check_if(NULL == g_service_h);

	ret = ua_service_remove_user(g_service_h, g_user_h);

	msg(" - ua_service_remove_user() ret: [0x%X] [%s]",
			ret, uat_get_error_str(ret));

	return RET_SUCCESS;
}

static struct menu_data menu_ua_service_add_user[] = {
	{ "-", "service",
		NULL, NULL, g_service_str},
	{ "-", "user account",
		NULL, NULL, g_user_account_str},
	{ "-", "user name",
		NULL, NULL, g_user_name_str},
	{ "1", "run", NULL,
		run_ua_service_add_user, NULL },
	{ NULL, NULL, },
};

static struct menu_data menu_ua_service_remove_user[] = {
	{ "-", "service",
		NULL, NULL, g_service_str},
	{ "-", "user account",
		NULL, NULL, g_user_account_str},
	{ "-", "user name",
		NULL, NULL, g_user_name_str},
	{ "1", "run", NULL,
		run_ua_service_remove_user, NULL },
	{ NULL, NULL, },
};

struct menu_data menu_ua_users[] = {
	{ "1", "ua_user_get_default_user",
		NULL, run_ua_get_default_user, NULL },
	{ "2", "ua_user_create",
		NULL, run_ua_user_create, NULL },
	{ "3", "ua_user_destroy",
		NULL, run_ua_user_destroy, NULL },
	{ "4", "ua_user_set_account",
		menu_ua_user_account, NULL, g_user_account_str },
	{ "5", "ua_user_set_name",
		menu_ua_user_name, NULL, g_user_name_str },
	{ "6", "ua_user_add",
		NULL, run_ua_user_add, NULL },
	{ "7", "ua_user_remove",
		NULL, run_ua_user_remove, NULL},
	{ "8", "ua_user_foreach_added",
		NULL, run_ua_user_foreach, NULL },
	{ "9", "ua_user_get_handle_by_account",
		menu_ua_user_get_handle_by_account, NULL, NULL },
	{ "10", "ua_service_add_user",
		menu_ua_service_add_user, NULL, NULL },
	{ "11", "ua_service_remove_user",
		menu_ua_service_remove_user, NULL, NULL },
	{ "12", "ua_service_foreach_added_users",
		NULL, run_ua_service_foreach_added_users, NULL },
	{ NULL, NULL, },
};

static char data_choose_user[MENU_DATA_SIZE + 1] = {0,};

static int run_choose_user_list(MManager *mm, struct menu_data *menu)
{
	__clear_user_list();
	ua_user_foreach_added(__foreach_registered_user_cb, NULL);

	return RET_SUCCESS;
}

static int run_select_user(MManager *mm, struct menu_data *menu)
{
	GSList *iter = user_list;

	int id = 0;
	int selected_id = 0;

	if (strlen(data_choose_user))
		selected_id = (unsigned char)strtol(data_choose_user, NULL, 10);

	if (selected_id <= 0) {
		msg("Please select user first");
		return RET_SUCCESS;
	}

	while (NULL != iter) {
		GSList *next = iter->next;
		ua_user_h *handle = iter->data;
		if (handle && ++id == selected_id) {
			g_user_h = handle; /* Make selected user as a current one */
			update_user_info(); /* Update user info. */
			msg("Select user done");
			return RET_SUCCESS;
		}
		iter = next;
	}

	return RET_SUCCESS;
}

struct menu_data menu_sel_user[] = {
	{ "1", "User list", NULL, run_choose_user_list, data_choose_user},
	{ "2", "Apply", NULL, run_select_user , NULL },
	{ NULL, NULL, },
};

